/*
 Copyright (c) 2015-present, salesforce.com, inc. All rights reserved.
 
 Redistribution and use of this software in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright notice, this list of conditions
 and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of
 conditions and the following disclaimer in the documentation and/or other materials provided
 with the distribution.
 * Neither the name of salesforce.com, inc. nor the names of its contributors may be used to
 endorse or promote products derived from this software without specific prior written
 permission of salesforce.com, inc.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef CoreSalesforceSDK_CSFAvailability_h
#define CoreSalesforceSDK_CSFAvailability_h

#define CSF_VERSION_23_0 2300
#define CSF_VERSION_24_0 2400
#define CSF_VERSION_25_0 2500
#define CSF_VERSION_26_0 2600
#define CSF_VERSION_27_0 2700
#define CSF_VERSION_28_0 2800
#define CSF_VERSION_29_0 2900
#define CSF_VERSION_30_0 3000
#define CSF_VERSION_31_0 3100
#define CSF_VERSION_32_0 3200
#define CSF_VERSION_33_0 3300

#define CSF_AVAILABLE_API(_version) __CSF_AVAILABLE_INTERNAL_VERSION_##_version
#define CSF_UNSUPPORTED_API_BEGIN(_version) __CSF_UNSUPPORTED_API_BEGIN(CSF_VERSION_##_version)
#define CSF_UNSUPPORTED_API_END(_version) __CSF_UNSUPPORTED_API_END(CSF_VERSION_##_version)
#define CSF_VERSION CSF_VERSION_32_0

#define CSF_API(_version) CSF_VERSION >= CSF_VERSION_##_version

#ifdef DEBUG
#define __CSF_AVAILABLE_INTERNAL_AVAILABLE \
__attribute__((visibility("default")))
#define __CSF_AVAILABLE_INTERNAL_UNAVAILABLE  \
__attribute__((visibility("default"),deprecated("Not supported in the production Chatter Connect API")))
#else
#define __CSF_AVAILABLE_INTERNAL_AVAILABLE \
__attribute__((visibility("default")))
#define __CSF_AVAILABLE_INTERNAL_UNAVAILABLE \
__attribute__((visibility("default"),unavailable("Not supported in the production Chatter Connect API")))
#endif

#if CSF_VERSION >= CSF_VERSION_23_0
#define __CSF_AVAILABLE_INTERNAL_VERSION_23_0 __CSF_AVAILABLE_INTERNAL_AVAILABLE
#else
#define __CSF_AVAILABLE_INTERNAL_VERSION_23_0 __CSF_AVAILABLE_INTERNAL_UNAVAILABLE
#endif

#if CSF_VERSION >= CSF_VERSION_24_0
#define __CSF_AVAILABLE_INTERNAL_VERSION_24_0 __CSF_AVAILABLE_INTERNAL_AVAILABLE
#else
#define __CSF_AVAILABLE_INTERNAL_VERSION_24_0 __CSF_AVAILABLE_INTERNAL_UNAVAILABLE
#endif

#if CSF_VERSION >= CSF_VERSION_25_0
#define __CSF_AVAILABLE_INTERNAL_VERSION_25_0 __CSF_AVAILABLE_INTERNAL_AVAILABLE
#else
#define __CSF_AVAILABLE_INTERNAL_VERSION_25_0 __CSF_AVAILABLE_INTERNAL_UNAVAILABLE
#endif

#if CSF_VERSION >= CSF_VERSION_26_0
#define __CSF_AVAILABLE_INTERNAL_VERSION_26_0 __CSF_AVAILABLE_INTERNAL_AVAILABLE
#else
#define __CSF_AVAILABLE_INTERNAL_VERSION_26_0 __CSF_AVAILABLE_INTERNAL_UNAVAILABLE
#endif

#if CSF_VERSION >= CSF_VERSION_27_0
#define __CSF_AVAILABLE_INTERNAL_VERSION_27_0 __CSF_AVAILABLE_INTERNAL_AVAILABLE
#else
#define __CSF_AVAILABLE_INTERNAL_VERSION_27_0 __CSF_AVAILABLE_INTERNAL_UNAVAILABLE
#endif

#if CSF_VERSION >= CSF_VERSION_28_0
#define __CSF_AVAILABLE_INTERNAL_VERSION_28_0 __CSF_AVAILABLE_INTERNAL_AVAILABLE
#else
#define __CSF_AVAILABLE_INTERNAL_VERSION_28_0 __CSF_AVAILABLE_INTERNAL_UNAVAILABLE
#endif

#if CSF_VERSION >= CSF_VERSION_29_0
#define __CSF_AVAILABLE_INTERNAL_VERSION_29_0 __CSF_AVAILABLE_INTERNAL_AVAILABLE
#else
#define __CSF_AVAILABLE_INTERNAL_VERSION_29_0 __CSF_AVAILABLE_INTERNAL_UNAVAILABLE
#endif

#if CSF_VERSION >= CSF_VERSION_30_0
#define __CSF_AVAILABLE_INTERNAL_VERSION_30_0 __CSF_AVAILABLE_INTERNAL_AVAILABLE
#else
#define __CSF_AVAILABLE_INTERNAL_VERSION_30_0 __CSF_AVAILABLE_INTERNAL_UNAVAILABLE
#endif

#if CSF_VERSION >= CSF_VERSION_31_0
#define __CSF_AVAILABLE_INTERNAL_VERSION_31_0 __CSF_AVAILABLE_INTERNAL_AVAILABLE
#else
#define __CSF_AVAILABLE_INTERNAL_VERSION_31_0 __CSF_AVAILABLE_INTERNAL_UNAVAILABLE
#endif

#if CSF_VERSION >= CSF_VERSION_32_0
#define __CSF_AVAILABLE_INTERNAL_VERSION_32_0 __CSF_AVAILABLE_INTERNAL_AVAILABLE
#else
#define __CSF_AVAILABLE_INTERNAL_VERSION_32_0 __CSF_AVAILABLE_INTERNAL_UNAVAILABLE
#endif

#if CSF_VERSION >= CSF_VERSION_33_0
#define __CSF_AVAILABLE_INTERNAL_VERSION_33_0 __CSF_AVAILABLE_INTERNAL_AVAILABLE
#else
#define __CSF_AVAILABLE_INTERNAL_VERSION_33_0 __CSF_AVAILABLE_INTERNAL_UNAVAILABLE
#endif

#endif
